import { join } from 'path';
import { defineNuxtModule, createResolver, addPlugin, addComponent, addAutoImport, extendPages, addTemplate } from '@nuxt/kit';
import { cleanDoubleSlashes } from 'ufo';
import { isRepositoryEndpoint, getRepositoryName } from '@prismicio/client';
import * as prismicVue from '@prismicio/vue';
import { existsSync } from 'fs';
import consola from 'consola';

const name = "@nuxtjs/prismic";
const version = "3.0.0-alpha.3";

const fileExists = (path, extensions = ["js", "ts"]) => {
  if (!path) {
    return null;
  } else if (existsSync(path)) {
    return path;
  }
  const extension = extensions.find((extension2) => existsSync(`${path}.${extension2}`));
  return extension ? `${path}.${extension}` : null;
};

const logger = consola.withScope("nuxt:prismic");

const module = defineNuxtModule({
  meta: {
    name: name,
    version: version,
    configKey: "prismic",
    compatibility: { nuxt: "^3.0.0" }
  },
  defaults: (nuxt) => ({
    endpoint: "",
    clientConfig: {},
    client: cleanDoubleSlashes(`~/${nuxt.options.dir.app}/prismic/client`),
    linkResolver: cleanDoubleSlashes(`~/${nuxt.options.dir.app}/prismic/linkResolver`),
    htmlSerializer: void 0,
    injectComponents: true,
    components: {},
    preview: "/preview"
  }),
  hooks: {},
  setup(mergedOptions, nuxt) {
    var _a, _b, _c;
    if (!mergedOptions.endpoint) {
      logger.warn("Options `endpoint` is required, disabling module...");
      return;
    }
    const resolver = createResolver(import.meta.url);
    nuxt.options.build.transpile.push(resolver.resolve("runtime"), "@prismicio/vue");
    const addUserFileWithUndefinedFallback = (filename, path, extensions = ["js", "ts"]) => {
      const resolvedFilename = `prismic/${filename}.ts`;
      const resolvedPath = path ? path.replace(/^(~~|@@)/, nuxt.options.rootDir).replace(/^(~|@)/, nuxt.options.srcDir) : void 0;
      const maybeUserFile = fileExists(resolvedPath, extensions);
      if (maybeUserFile) {
        logger.info(`Using user-defined \`${filename}\` at \`${maybeUserFile.replace(nuxt.options.srcDir, "~").replace(/\\/g, "/")}\``);
        addTemplate({
          filename: resolvedFilename,
          src: maybeUserFile
        });
      } else {
        addTemplate({
          filename: resolvedFilename,
          getContents: () => "export default undefined"
        });
      }
    };
    addUserFileWithUndefinedFallback("client", mergedOptions.client);
    addUserFileWithUndefinedFallback("linkResolver", mergedOptions.linkResolver);
    (_a = nuxt.options).publicRuntimeConfig || (_a.publicRuntimeConfig = {});
    nuxt.options.publicRuntimeConfig[name] = mergedOptions;
    addPlugin(resolver.resolve("runtime/plugin"));
    if (mergedOptions.injectComponents) {
      [
        "PrismicEmbed",
        "PrismicImage",
        "PrismicLink",
        "PrismicText",
        "PrismicRichText",
        "SliceZone"
      ].forEach((component) => {
        addComponent({
          name: component,
          export: component,
          filePath: "@prismicio/vue"
        });
      });
    }
    const composableAutoImports = Object.keys(prismicVue).filter((key) => key.startsWith("use")).map((key) => {
      return {
        name: key,
        as: key,
        from: "@prismicio/vue"
      };
    });
    addAutoImport(composableAutoImports);
    if (mergedOptions.preview) {
      const maybeUserPreviewPage = fileExists(join(nuxt.options.srcDir, nuxt.options.dir.pages, mergedOptions.preview), ["js", "ts", "vue"]);
      if (maybeUserPreviewPage) {
        logger.info(`Using user-defined preview page at \`${maybeUserPreviewPage.replace(join(nuxt.options.srcDir), "~").replace(/\\/g, "/")}\`, available at \`${mergedOptions.preview}\``);
      } else {
        logger.info(`Using default preview page, available at \`${mergedOptions.preview}\``);
        extendPages((pages) => {
          pages.unshift({
            name: "prismic-preview",
            path: mergedOptions.preview,
            file: resolver.resolve("runtime/preview.vue")
          });
        });
      }
      const repositoryName = isRepositoryEndpoint(mergedOptions.endpoint) ? getRepositoryName(mergedOptions.endpoint) : mergedOptions.endpoint;
      (_b = nuxt.options).head || (_b.head = {});
      (_c = nuxt.options.head).script || (_c.script = []);
      nuxt.options.head.script.push({
        src: `https://static.cdn.prismic.io/prismic.min.js?repo=${repositoryName}&new=true`
      });
    }
  }
});

export { module as default };
